-- Settings are now modified under Interface Addon panel in game.

-- Default options that are loaded the first time the mod is loaded in the game
CEPetBar_Options = {
	["ChannelingBar"] = {
		["R"] = 1.0,
		["G"] = 0.7,
		["B"] = 0.0,
		["O"] = 1.0
	},
	["CastingBar"] = {
		["R"] = 1.0,
		["G"] = 0.7,
		["B"] = 0.0,
		["O"] = 0.5
	},
	["CastingText"] = {
		["R"] = 1.0,
		["G"] = 0.82,
		["B"] = 0.0
	},
	["Exp"] = {
		["R"] = 1.0,
		["G"] = 1.0,
		["B"] = 0.2,
		["O"] = 1.0
	},
	["DisplayTime"] = 1,
	["DisplayExp"] = 1,
	["DisplayCast"] = 1,
	["ShowSpellName"] = 1,
	["FontSize"] = 8,
	["HoverPattern"] = "&percentxp% XP",
	["NoHoverPattern"] = "&Petname",
	["PercentEnabled"] = nil,
	["PercentPattern"] = "&remainingxppercent%",
	["Version"] = 1
}

--------------------------------------------------------------------------------
-- Local variables

-- variables to store the original Pet name color
local petOrigR, petOrigG, petOrigB = nil;

-- casting related variables
CEPB_PetCasting = 0;
CEPB_PetChanneling = 0;
local petSpell, petStartTime, petEndTime, petFont, petOldFontSize, petOtherStuff = nil;

local IsPlayerInVehicle_Pet = 0;
local didPetGoPoof = 0;

--------------------------------------------------------------------------------
-- Table management functions

function GetPetVal(option, table)
	if (table) then
		return CEPetBar_Options[table][option];
	else
		return CEPetBar_Options[option];
	end
end

function SetPetVal(option, value, table)
	if (table) then
		CEPetBar_Options[table][option] = value;
	else
		CEPetBar_Options[option] = value;
	end
end

--------------------------------------------------------------------------------
-- Event / Texture Registrations

function CEPetBar_SetPetTexture()
	local texture = PetFrameTexture:GetTexture();
	if (texture ~= "Interface\\AddOns\\CEPlayerBar\\textures\\PetReplacementFrame") then
		CEPB_OldPetTexture = texture;
	end
	PetFrameTexture:SetTexture("Interface\\AddOns\\CEPlayerBar\\textures\\PetReplacementFrame.tga");
	PetName:SetFont(petFont, GetPetVal("FontSize"), petOtherStuff);
end

function CEPetBar_UnloadPetTexture()
	PetFrameTexture:SetTexture(CEPB_OldPetTexture);
	PetName:SetFont(petFont, petOldFontSize, petOtherStuff);
end

function CEPetBar_RegisterPetEvents(self, excludeRequired)
	if (excludeRequired == false) then
		self:RegisterEvent("UNIT_PET");
		self:RegisterEvent("UNIT_PET_EXPERIENCE");
	end

	-- This section is from Blizzard's PetFrame.lua file.  Some of the numbers need to be adjusted because
	-- their bar is pushed up against the player frame in some cases.
	-- I changed self to PetFrame and seperated the classes completely in case they needed individual adjustments.
	local _, class = UnitClass("player");
	if ( class == "DEATHKNIGHT") then	--Death Knights need the Pet frame moved down for their Runes and Druids need it moved down for the secondary power bar.
		PetFrame:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 60, -80); -- adjusted down 5 points
	elseif ( class == "SHAMAN" ) then
		PetFrame:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 60, -100);
	elseif ( class == "DRUID" ) then
		-- druid eclipse bar is partially covering mana bar... correct it.
		local form  = GetShapeshiftFormID();
		if (form == MOONKIN_FORM or not form) then
			EclipseBarFrame:SetPoint("TOP", PlayerFrame, "BOTTOM", "48", "35");
		end
		PetFrame:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 60, -92); -- Adjusted up 8 points
	elseif ( class == "WARLOCK" ) then
		PetFrame:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 60, -80);
	elseif ( class == "PALADIN" ) then
		PetFrame:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 60, -90);
	end

	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("VARIABLES_LOADED");

	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_START");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_STOP");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_FAILED");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_DELAYED");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");

	self:RegisterEvent("UNIT_SPELLCAST_START");
	self:RegisterEvent("UNIT_SPELLCAST_STOP");
	self:RegisterEvent("UNIT_SPELLCAST_FAILED");
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	self:RegisterEvent("UNIT_SPELLCAST_DELAYED");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");

	CEPetBar_SetPetTexture();
end

function CEPetBar_UnregisterPetEvents(self, excludeRequired)
	if (excludeRequired == false) then
		self:UnregisterEvent("UNIT_PET");
		self:UnregisterEvent("UNIT_PET_EXPERIENCE");
	end

	self:UnregisterEvent("PLAYER_ENTERING_WORLD");
	self:UnregisterEvent("VARIABLES_LOADED");

	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");

	self:UnregisterEvent("UNIT_SPELLCAST_START");
	self:UnregisterEvent("UNIT_SPELLCAST_STOP");
	self:UnregisterEvent("UNIT_SPELLCAST_FAILED");
	self:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	self:UnregisterEvent("UNIT_SPELLCAST_DELAYED");
	self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
end

--------------------------------------------------------------------------------
-- State management functions

function CEPetBar_Load()
	CEPetBar_RegisterPetEvents(_G["CEPetBar_Frame"], false);
	CEPetBar:Show();
	CEPetBar_ShowXP();
end

function CEPetBar_Unload()
	CEPetBar_UnregisterPetEvents(_G["CEPetBar_Frame"], false);
	CEPetBar_UnloadPetTexture();
	CEPetBar:Hide();
end

function CEPetBar_ManageCurrentState()
	local isCastEnabled = GetPetVal("DisplayCast");
	local isExpEnabled = GetPetVal("DisplayExp");
	if (isCastEnabled == 0 and isExpEnabled == 0) then
		CEPetBar_Unload();
	else
		CEPetBar_Load();
	end
end

--------------------------------------------------------------------------------
-- Load

function CEPetBar_OnLoad (self)
	petOrigR, petOrigG, petOrigB = PetName:GetTextColor();
	petFont, petOldFontSize, petOtherStuff = PetName:GetFont();

	if (GetPlayerVal("DisplayPetBar") == 1) then
		CEPetBar_RegisterPetEvents(self, false);
		CEPB_PetCasting = 0;
	end
end

--------------------------------------------------------------------------------
-- Event handling

function CEPetBar_OnEvent (self, event, ...)
	IsPlayerInVehicle_Pet = UnitInVehicle("player");
	if (IsPlayerInVehicle_Pet == nil) then
		IsPlayerInVehicle_Pet = 0;
	end

	if (GetPlayerVal("DisplayPetBar") == 1 and IsPlayerInVehicle_Pet == 0) then

		local isEligible = 0;
		local type, _, _, _, _, _, _, _, _ = ...;

		-- Check vehicle status
		if (type == "pet") then
			isEligible = 1;
		end

		if ( event == "UNIT_SPELLCAST_START" and isEligible == 1 and GetPetVal("DisplayCast") == 1 ) then
			-- Get spell information
			petSpell, _, _, _, petStartTime, petEndTime = UnitCastingInfo(type);
			petStartTime = petStartTime / 1000;
			petEndTime = petEndTime / 1000;

			-- Set cast bar properties
			CEPetBar:SetStatusBarColor(GetPetVal("R", "CastingBar"), GetPetVal("G", "CastingBar"), GetPetVal("B", "CastingBar"), GetPetVal("O", "CastingBar"));
			CEPetBar:SetMinMaxValues(petStartTime, petEndTime);
			CEPetBar:SetValue(petStartTime);

			-- Set spell text properties
			PetName:SetFont(petFont, GetPetVal("FontSize"), petOtherStuff);
			PetName:SetTextColor(GetPetVal("R", "CastingText"), GetPetVal("G", "CastingText"), GetPetVal("B", "CastingText"));

			-- Show spell information
			if (GetPetVal("DisplayTime") == 1 and GetPetVal("ShowSpellName") == 1) then
				if (math.floor((petEndTime - petStartTime) / 1000) == (petEndTime - petStartTime) / 1000) then
					PetName:SetText( petSpell .. " " .. ((petEndTime - petStartTime) / 1000) .. ".0");
				else
					PetName:SetText( petSpell .. " " .. (petEndTime - petStartTime) / 1000);
				end

			elseif (GetPetVal("DisplayTime") == 1 and GetPetVal("ShowSpellName") == 0) then
				if (math.floor((petEndTime - petStartTime) / 1000) == (petEndTime - petStartTime) / 1000) then
					PetName:SetText(((petEndTime - petStartTime) / 1000) .. ".0");
				else
					PetName:SetText((petEndTime - petStartTime) / 1000);
				end

			elseif (GetPetVal("DisplayTime") == 0 and GetPetVal("ShowSpellName") == 1) then
				PetName:SetText( petSpell );
			end

			CEPB_PetCasting = 1;

		elseif ( event == "UNIT_SPELLCAST_STOP" and isEligible == 1 and GetPetVal("DisplayCast") == 1 ) then
			if (CEPB_PetCasting == 1) then
				CEPB_PetCasting = 0;
				PetName:SetFont(petFont, petOldFontSize, petOtherStuff);
				CEPetBar_ShowXP();
				CEPetBar_RealUpdate();
				return;
			end

		elseif (event == "UNIT_SPELLCAST_CHANNEL_STOP" and isEligible == 1 and GetPetVal("DisplayCast") == 1 ) then
			if (CEPB_PetChanneling == 1) then
				CEPB_PetChanneling = 0;
				PetName:SetFont(petFont, petOldFontSize, petOtherStuff);
				CEPetBar_ShowXP();
				CEPetBar_RealUpdate();
				return;
			end

		elseif ( (event == "UNIT_SPELLCAST_FAILED" or event == "SPELLCAST_INTERRUPTED") and isEligible == 1 and GetPetVal("DisplayCast") == 1 ) then
			CEPB_PetCasting = 0;
			CEPB_PetChanneling = 0;
			CEPetBar_ShowXP();
			CEPetBar_RealUpdate();
			return;

		elseif ( event == "UNIT_SPELLCAST_DELAYED"  and isEligible == 1 and GetPetVal("DisplayCast") == 1 ) then
			local _, _, _, _, _, newEndTime = UnitCastingInfo(type);
			newEndTime = newEndTime / 1000;
			petEndTime = newEndTime;
			CEPetBar:SetMinMaxValues(petStartTime, petEndTime);

			if (GetPetVal("DisplayTime") == 1) then
				if (math.floor(math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10) == math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10) then
					PetName:SetText((math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10) .. ".0");
				else
					PetName:SetText(math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10);
				end
			end

		elseif ( event == "UNIT_SPELLCAST_CHANNEL_START" and isEligible == 1 and GetPetVal("DisplayCast") == 1 ) then
			-- Get spell information
			petSpell, _, _, _, petStartTime, petEndTime = UnitChannelInfo(type);
			petStartTime = petStartTime / 1000;
			petEndTime = petEndTime / 1000;

			-- Set cast bar properties
			CEPetBar:SetMinMaxValues(petStartTime, petEndTime);
			CEPetBar:SetValue(petEndTime);
			CEPetBar:SetStatusBarColor(GetPetVal("R", "ChannelingBar"), GetPetVal("G", "ChannelingBar"), GetPetVal("B", "ChannelingBar"), GetPetVal("O", "ChannelingBar"));

			-- Set spell text properties
			PetName:SetFont(petFont, GetPetVal("FontSize"), petOtherStuff);
			PetName:SetTextColor(GetPetVal("R", "CastingText"), GetPetVal("G", "CastingText"), GetPetVal("B", "CastingText"));

			-- Show spell information
			if (GetPetVal("DisplayTime") == 1 and GetPetVal("ShowSpellName") == 1) then
				if (math.floor((petEndTime - petStartTime) / 1000) == (petEndTime - petStartTime) / 1000) then
					PetName:SetText( petSpell .. " " .. ((petEndTime - petStartTime) / 1000) .. ".0");
				else
					PetName:SetText( petSpell .. " " .. (petEndTime - petStartTime) / 1000);
				end

			elseif (GetPetVal("DisplayTime") == 1 and GetPetVal("ShowSpellName") == 0) then
				if (math.floor((petEndTime - petStartTime) / 1000) == (petEndTime - petStartTime) / 1000) then
					PetName:SetText(((petEndTime - petStartTime) / 1000) .. ".0");
				else
					PetName:SetText((petEndTime - petStartTime) / 1000);
				end

			elseif (GetPetVal("DisplayTime") == 0 and GetPetVal("ShowSpellName") == 1) then
				PetName:SetText( petSpell );
			end

			CEPB_PetCasting = 0;
			CEPB_PetChanneling = 1;

		elseif ( event == "UNIT_SPELLCAST_CHANNEL_UPDATE" and isEligible == 1 and GetPetVal("DisplayCast") == 1 ) then
			local _, _, _, _, _, newEndTime = UnitChannelInfo(type);
			if (newEndTime ~= nil) then
				newEndTime = newEndTime / 1000;
				if ( GetTime() >= newEndTime ) then
					CEPB_PetChanneling = 0;
					CEPetBar_ShowXP();
					CEPetBar_RealUpdate();
					return;

				elseif ( CEPB_PetChanneling == 1 ) then
					local durationShift = petEndTime - newEndTime;
					petEndTime = newEndTime;
					CEPetBar:SetValue( CEPetBar:GetValue() - durationShift );
					if (GetPetVal("DisplayTime") == 1) then
						if (math.floor(math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10) == math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10) then
							PetName:SetText( petSpell .. " " .. (math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10) .. ".0");
						else
							PetName:SetText( petSpell .. " " .. math.floor((CEPetBar:GetValue() * 10) - (petStartTime * 10))/10);
						end
					end
				end
			end

		elseif (event == "UNIT_PET_EXPERIENCE" and CEPB_PetCasting ~= 1) then
			CEPetBar_ShowXP();
			CEPetBar_RealUpdate();
			CEPB_PetCasting = 0;

		elseif (event == "UNIT_PET" and CEPB_PetCasting ~= 1) then
			CEPB_PetCasting = 0;

			local isItAlive = UnitExists("pet");
			if (isItAlive == nil) then
				isItAlive = 0;
			end

			if (isItAlive == 1 or IsPlayerInVehicle_Pet == 0) then
				didPetGoPoof = 0;
			elseif (isItAlive == 0 or IsPlayerInVehicle_Pet == 1) then
				didPetGoPoof = 1;
			end

			if (didPetGoPoof == 0) then
				CEPetBar_ShowXP();
				CEPetBar_RealUpdate();
				CEPetBar_RegisterPetEvents(self, true);
			else
				CEPetBar_UnregisterPetEvents(self, true);
			end
		end
	end
end

function CEPetBar_ProcessBarSettingsWhileCasting()
	if (CEPB_PetChanneling == 1) then
		CEPetBar:SetStatusBarColor(GetPetVal("R", "ChannelingBar"), GetPetVal("G", "ChannelingBar"), GetPetVal("B", "ChannelingBar"), GetPetVal("O", "ChannelingBar"));
		PetName:SetTextColor(GetPetVal("R", "CastingText"), GetPetVal("G", "CastingText"), GetPetVal("B", "CastingText"));
	elseif (CEPB_PetCasting == 1) then
		CEPetBar:SetStatusBarColor(GetPetVal("R", "CastingBar"), GetPetVal("G", "CastingBar"), GetPetVal("B", "CastingBar"), GetPetVal("O", "CastingBar"));
		PetName:SetTextColor(GetPetVal("R", "CastingText"), GetPetVal("G", "CastingText"), GetPetVal("B", "CastingText"));
	end
end

--------------------------------------------------------------------------------
-- Handle clicking on new bar (extend pet bar basically)

function CEPetBar_OnClick(self, button)
	if ( SpellIsTargeting() and button == "RightButton" ) then
		SpellStopTargeting();
		return;
	end

	if ( button == "LeftButton" ) then
		if ( SpellIsTargeting() ) then
			-- SpellTargetUnit("pet");
		elseif ( CursorHasItem() ) then
			AutoEquipCursorItem();
		else
			-- TargetUnit("pet");
		end
	else
		ToggleDropDownMenu(1, nil, PetFrameDropDown, "PetFrame", 106, 27);
		return;
	end
end

--------------------------------------------------------------------------------
-- Bar state functions

function CEPetBar_Update(self)
	IsPlayerInVehicle_Pet = UnitInVehicle("player");
	if (IsPlayerInVehicle_Pet == nil) then
		IsPlayerInVehicle_Pet = 0;
	end

	CEPetBar_RealUpdate();
end

function CEPetBar_RealUpdate()
	if (GetPlayerVal("DisplayPetBar") == 1 and IsPlayerInVehicle_Pet == 0) then
		if (UnitExists("pet") == 1) then
			if (CEPB_PetCasting ~= 1) then
				if (CEPetBar_IsHovering == 1) then
					PetName:SetText(CEPetBar_Process(GetPetVal("HoverPattern")));
				else
					local tempVal = UnitInVehicle("player");
					if (tempVal == nil) then
						tempVal = 0;
					end

					if (tempVal == 0) then
						PetName:SetText(CEPetBar_Process(GetPetVal("NoHoverPattern")));
					end
				end

				if (GetPetVal("PercentEnabled") == 1) then
					CEPetBarPercent:Show();
					CEPetBarPercent:SetText(CEPetBar_Process(GetPetVal("PercentPattern")));
				else
					CEPetBarPercent:Hide();
				end

				if (GetPetVal("DisplayTime") == 1) then
					PetName:SetTextColor(petOrigR, petOrigG, petOrigB);
				end
			end

			CEPetBar_SetPetTexture();
		end
	end
end

function CEPetBar_ShowXP()
	local currentXP, maxXP = GetPetExperience();
	if (maxXP == 0) then
		maxXP = 1;
		currentXP = 0;
	end
	if (GetPlayerVal("DisplayPetBar") == 1 and GetPetVal("DisplayExp") == 1 and IsPlayerInVehicle_Pet == 0) then
		CEPetBar:SetMinMaxValues(min(0, currentXP), maxXP);
		CEPetBar:SetValue(currentXP);
		CEPetBar:SetStatusBarColor(GetPetVal("R", "Exp"), GetPetVal("G", "Exp"), GetPetVal("B", "Exp"), GetPetVal("O", "Exp"));
	elseif (GetPlayerVal("DisplayPetBar") == 1 and GetPetVal("DisplayExp") == 0 and IsPlayerInVehicle_Pet == 0) then
		CEPetBar:SetMinMaxValues(min(0, currentXP), maxXP);
		CEPetBar:SetValue(0);
	end
end

function CEPetBar_UpdateTimers(self)
	if (GetPlayerVal("DisplayPetBar") == 1 and IsPlayerInVehicle_Pet == 0) then
		if (CEPB_PetCasting == 1 or CEPB_PetChanneling == 1) then
			local time = GetTime();
			if ( time  > petEndTime ) then
				time  = petEndTime;
			end

			if ( CEPB_PetCasting == 1 ) then
				CEPetBar:SetValue(time);
			elseif ( CEPB_PetChanneling == 1 ) then
				local barValue = petStartTime + (petEndTime - time);
				CEPetBar:SetValue(barValue);
			end

			if (GetPetVal("DisplayTime") == 1 and GetPetVal("ShowSpellName") == 1) then
				if (math.floor(math.floor((petEndTime * 10) - (time * 10))/10) == math.floor((petEndTime * 10) - (time * 10))/10) then
					PetName:SetText( petSpell .. " " .. (math.floor((petEndTime * 10) - (time * 10))/10)..".0");
				else
					PetName:SetText( petSpell .. " " .. math.floor((petEndTime * 10) - (time * 10))/10);
				end

			elseif (GetPetVal("DisplayTime") == 1 and GetPetVal("ShowSpellName") == 0) then
				if (math.floor(math.floor((petEndTime * 10) - (time * 10))/10) == math.floor((petEndTime * 10) - (time * 10))/10) then
					PetName:SetText((math.floor((petEndTime * 10) - (time * 10))/10)..".0");
				else
					PetName:SetText(math.floor((petEndTime * 10) - (time * 10))/10);
				end

			elseif (GetPetVal("DisplayTime") == 0 and GetPetVal("ShowSpellName") == 1) then
				PetName:SetText( petSpell );
			end
		end
	end
	
	-- I set the texture here because when the Interface Addons page is closed, the texture resets
	-- I spent over an hour trying to find out what did it without luck.  I tried to use the
	-- UNIT_POWER event which triggers every 2 seconds but that isn't triggered when the bar is full.
	-- As a debugging note, RegisterAllEvents showed no event being raised when the Addon page is
	-- closed so I'd have to isolate the function calling it and securely hook it to directly
	-- address the problem.
	if (GetPlayerVal("DisplayPetBar") == 1 and IsPlayerInVehicle_Pet == 0 and UnitExists("pet") == 1) then
		CEPetBar_SetPetTexture();
	end
end

--------------------------------------------------------------------------------
-- Process variables

function CEPetBar_Process(val)
	local currentXP, maxXP = GetPetExperience();
	local remainingXpPer;

	if (maxXP == 0) then
		remainingXpPer = 0;
	else
		remainingXpPer = floor((maxXP - currentXP)/maxXP*100);
	end

	if (maxXP == 0) then
		currentXP = 0;
		maxXP = 1;
	end

	val = gsub(val, "&Petname", UnitName("pet"));
	val = gsub(val, "&percentxp", floor(currentXP/maxXP*100));
	val = gsub(val, "&remainingxppercent", remainingXpPer);
	val = gsub(val, "&currentxp", currentXP);
	val = gsub(val, "&maxxp", maxXP);
	val = gsub(val, "&remainingxp", maxXP-currentXP);
	return val;
end