-- Structure of code from SCT mostly...

--The Options Page functions that process the load

local _G = _G;
local pairs = pairs;

CEPB.OptionFrames = { ["CEPBOptionsFrame_Misc1"] = "CEPBOptions_PlayerFrame",
                     ["CEPBOptionsFrame_Misc2"] = "CEPBOptions_PetFrame"}

------------------------------
-- Sort table requirements
local table_sort = _G.table.sort;
local tinsert = function(t,v)
  t[#t+1] = v
end
                     
------------------------------
---Sort a table by its keys.
---stolen from SCT who stole it from http://www.lua.org/pil/19.3.html
function CEPB:PairsByKeys(t, f)
  local a = {}
  for n in pairs(t) do tinsert(a, n) end
  table_sort(a, f)
  local i = 0      -- iterator variable
  local iter = function ()   -- iterator function
    i = i + 1
    if a[i] == nil then return nil
    else return a[i], t[a[i]]
    end
  end
  return iter
end

---------------------
--Init a Custom Dropdown
function CEPB:CustomDropDown_Initialize(ddl)
	local info, index, value, key;
	for index, value in pairs(CEPB.OPTIONS.FrameCustomSelections) do
		if (ddl:GetName() == "CEPBOptionsFrame_CustomSelection"..value.index) then
			for k, v in self:PairsByKeys(value.table) do
				info = UIDropDownMenu_CreateInfo();
				info.text = v;
				info.func = CEPB.CustomDropDown_OnClick;
				info.arg1 = ddl;
				info.value = k;
				UIDropDownMenu_AddButton(info);
			end
			break
		end
	end
	
	-- Set default selected
	if (not ddl.didInitialPopulate) then
		UIDropDownMenu_SetSelectedID(ddl, 1);
		ddl.didInitialPopulate = true;
	end
end

----------------------
--Called when option page loads
function CEPB:OptionsFrame_OnShow()
	local option1, option2, option3, option4, option5, label, getvalue;
	local frame, swatch, sColor;

	--Header Options
	for key, value in pairs(CEPB.OPTIONS.FrameHeader) do
	    label = _G["CEPBOptionsFrame_Header"..value.index.."_Label"];
		if (label) then
			label:SetText(key);
			if (value.tooltipText) then
				label.tooltipText = value.tooltipText;
			end
		end
	end

	--Section Options
	for key, value in pairs(CEPB.OPTIONS.FrameSection) do
		label = _G["CEPBOptionsFrame_Section"..value.index.."_Label"]
		if (label) then
			label:SetText(key)
			if (value.tooltipText) then
				label.tooltipText = value.tooltipText;
			end
		end
	end

	-- Set CheckButton states
	for key, value in pairs(CEPB.OPTIONS.FrameCheckButtons) do
		option1 = _G["CEPBOptionsFrame_CheckButton"..value.index];
		text = _G["CEPBOptionsFrame_CheckButton"..value.index.."Text"];
		if (option1 and text) then
			if (value.CEPBType == "player") then
				option1:SetChecked(GetPlayerVal(value.CEPBVar));
			elseif (value.CEPBType == "pet") then
				option1:SetChecked(GetPetVal(value.CEPBVar));
			end
			option1.CEPBVar = value.CEPBVar;
			option1.CEPBType = value.CEPBType;
      		option1.tooltipText = value.tooltipText;
      		text:SetText(key);
		end
	end

	--Set Sliders
	local canSet;
	for key, value in pairs(CEPB.OPTIONS.FrameSliders) do
	    canSet = true;
		option1 = _G["CEPBOptionsFrame_Slider"..value.index.."Slider"];
		text = _G["CEPBOptionsFrame_Slider"..value.index.."SliderText"];
		option2 = _G["CEPBOptionsFrame_Slider"..value.index.."SliderLow"];
		option3 = _G["CEPBOptionsFrame_Slider"..value.index.."SliderHigh"];

		if (value.CEPBVar == "Opacity") then
		    canSet = false;
		else
		    if (value.CEPBType == "player") then
			    getvalue = GetPlayerVal(value.CEPBVar);
		    elseif (value.CEPBType == "pet") then
			    getvalue = GetPetVal(value.CEPBVar);
		    end
		end

		text:SetText(key);
		option1.CEPBLabel = key;
		-- Disable OnValueChanged hook temporarily so we can set the values without triggering the function
		option1.isLoading = true;
		option1:SetMinMaxValues(value.minValue, value.maxValue);
		if (canSet) then
		    -- this line must be set after setting min / max
		    option1:SetValue(getvalue);
		end
		option1.isLoading = false;
		option1:SetValueStep(value.valueStep);
		option1.tooltipText = value.tooltipText;
		option2:SetText(value.minText);
		option3:SetText(value.maxText);
		option1.CEPBVar = value.CEPBVar;
		option1.CEPBType = value.CEPBType;
	end
	
    -- Colors
    -- Only set variables, not colors.  Color setting takes place when
    -- the drop down value changes.
    for key, value in pairs(CEPB.OPTIONS.FrameColors) do
        frame = _G["CEPBOptionsFrame_Color"..value.index];
        swatch = _G["CEPBOptionsFrame_Color"..value.index.."NormalTexture"];
        local k = value.CEPBType;
        local s = swatch:GetName();
        local f = frame:GetName();
        frame.swatchFunc = function() self:OptionsFrame_SetColor(f, s, k) end
        frame.cancelFunc = function(x) self:OptionsFrame_CancelColor(f, s, k, x) end
        label:SetText(key);
    end

	--Custom Dropdowns
	for key, value in pairs(CEPB.OPTIONS.FrameCustomSelections) do
		option1 = _G["CEPBOptionsFrame_CustomSelection"..value.index];
		option2 = _G["CEPBOptionsFrame_CustomSelection"..value.index.."Label"];
		option1.CEPBVar = value.CEPBVar;
		option1.CEPBType = value.CEPBType;
		option1.lookup = value.table;
		option1.tooltipText = value.tooltipText;
		option2:SetText(key);
		
		-- Trigger programatic click on it so we hide the UI elements that
		-- shouldn't be shown on the load
		if (not option1.didInitialHide or UIDropDownMenu_GetSelectedValue(option1) == "A_Default") then
			CEPB:CustomDropDown_OnClick(option1, "A_Default");
			option1.didInitialHide = true;
		end
	end

	--Edit Boxes
	for key, value in pairs(CEPB.OPTIONS.EditBoxes) do
		frame = _G["CEPBOptionsFrame_EditBox"..value.index.."_EditBox"];
		label = _G["CEPBOptionsFrame_EditBox"..value.index.."_Label"];
		if (frame) then
			label:SetText(key);
			if (value.tooltipText) then
				frame.tooltipText = value.tooltipText;
			end
		end
	end
end

----------------------
--Open the color selector using show/hide
function CEPB:OpenColorPicker(button)
	CloseMenus();
	if ( not button ) then
		button = self;
	end
	ColorPickerFrame.func = button.swatchFunc;
	ColorPickerFrame:SetColorRGB(button.r, button.g, button.b);
	ColorPickerFrame.previousValues = {r = button.r, g = button.g, b = button.b, opacity = button.opacity};
	ColorPickerFrame.cancelFunc = button.cancelFunc;
	ColorPickerFrame:Show();
end

---------------------
--Set options on Blizzard Addon menu

function CEPB:MakeBlizzOptions()
	self:OptionsFrame_OnShow();

    CEPBOptions_PlayerFrame.name = CEPB.LOCALS.OPTION_HEADER1.name;
    CEPBOptions_PlayerFrame.parent = "CEPlayerBar";
    CEPBOptions_PlayerFrame.default = function() CEPB:Reset() end
    InterfaceOptions_AddCategory(CEPBOptions_PlayerFrame);

    CEPBOptions_PetFrame.name = CEPB.LOCALS.OPTION_HEADER2.name;
    CEPBOptions_PetFrame.parent = "CEPlayerBar";
    CEPBOptions_PetFrame.default = function() CEPB:Reset() end
    InterfaceOptions_AddCategory(CEPBOptions_PetFrame);
    
    -- Open the menu to show the children
    InterfaceOptionsFrame_OpenToCategory(CEPB.LOCALS.OPTION_HEADER1.name);
end